@@ -2,7 +2,7 @@
 /* -----------------------------------------------------------------------------------------------------------
 Software License for The Fraunhofer FDK AAC Codec Library for Android
 
- Copyright  1995 - 2012 Fraunhofer-Gesellschaft zur Frderung der angewandten Forschung e.V.
+ Copyright  1995 - 2014 Fraunhofer-Gesellschaft zur Frderung der angewandten Forschung e.V.
   All rights reserved.
 
  1.    INTRODUCTION
@@ -325,7 +325,6 @@ FDKaacEnc_prepareIntensityDecision(const FIXP_DBL    *sfbEnergyLeft,
 
       channelCorr[sfb + sfboffs] = FL2FXCONST_DBL(0.0f);
 
-      FDK_ASSERT(50 >= 49);
       /* max width of scalefactorband is 96; width's are always even */
       /* inv_n is scaled with factor 2 to compensate fMultDiv2() in subsequent loops */
       inv_n = GetInvInt((sfbOffset[sfb + sfboffs + 1] - sfbOffset[sfb + sfboffs])>>1);
@@ -450,23 +449,24 @@ FDKaacEnc_finalizeIntensityDecision(const FIXP_DBL *hrrErr,
                                     const INT       maxSfbPerGroup)
 {
   INT sfb,sfboffs, j;
-  INT startIsSfb = 0;
-  INT inIsBlock;
-  INT currentIsSfbCount;
-  FIXP_DBL overallHrrError;
   FIXP_DBL isScaleLast = FL2FXCONST_DBL(0.0f);
-  FIXP_DBL isRegionLoudness;
+  INT isStartValueFound = 0;
 
   for (sfboffs = 0; sfboffs < sfbCnt; sfboffs += sfbPerGroup) {
-    inIsBlock = 0;
-    currentIsSfbCount = 0;
-    overallHrrError = FL2FXCONST_DBL(0.0f);
-    isRegionLoudness = FL2FXCONST_DBL(0.0f);
+    INT startIsSfb = 0;
+    INT inIsBlock = 0;
+    INT currentIsSfbCount = 0;
+    FIXP_DBL overallHrrError = FL2FXCONST_DBL(0.0f);
+    FIXP_DBL isRegionLoudness = FL2FXCONST_DBL(0.0f);
+
     for (sfb = 0; sfb < maxSfbPerGroup; sfb++) {
       if (isMask[sfboffs + sfb] == 1) {
         if (currentIsSfbCount == 0) {
           startIsSfb = sfboffs + sfb;
+        }
+        if (isStartValueFound==0) {
           isScaleLast = realIsScale[sfboffs + sfb];
+          isStartValueFound = 1;
         }
         inIsBlock = 1;
         currentIsSfbCount++;
@@ -510,6 +510,14 @@ FDKaacEnc_finalizeIntensityDecision(const FIXP_DBL *hrrErr,
           for(j = startIsSfb; j <= sfboffs + sfb; j++) {
             isMask[j] = 0;
           }
+          isScaleLast = FL2FXCONST_DBL(0.0f);
+          isStartValueFound = 0;
+          for (j=0; j < startIsSfb; j++) {
+            if (isMask[j]!=0) {
+              isScaleLast = realIsScale[j];
+              isStartValueFound = 1;
+            }
+          }
         }
         currentIsSfbCount = 0;
         overallHrrError = FL2FXCONST_DBL(0.0f);
@@ -584,7 +592,6 @@ void FDKaacEnc_IntensityStereoProcessing(
   FIXP_DBL realIsScale[MAX_GROUPED_SFB];
   INTENSITY_PARAMETERS isParams;
   INT isMask[MAX_GROUPED_SFB];
-  INT lastValIs = 0, delta;
 
   FDKmemclear((void*)isBook,sfbCnt*sizeof(INT));
   FDKmemclear((void*)isMask,sfbCnt*sizeof(INT));
@@ -740,14 +747,6 @@ void FDKaacEnc_IntensityStereoProcessing(
         isScale[sfb+sfboffs] = (INT)(((realIsScale[sfb+sfboffs]>>1)+FL2FXCONST_DBL(0.5f/(1<<(REAL_SCALE_SF+LD_DATA_SHIFT+1))))>>(DFRACT_BITS-1-REAL_SCALE_SF-LD_DATA_SHIFT-1));
       }
 
-      delta = isScale[sfb+sfboffs] - lastValIs;
-      if (delta < -CODE_BOOK_SCF_LAV) {
-          isScale[sfb+sfboffs] = lastValIs - CODE_BOOK_SCF_LAV;
-      } else if (delta + CODE_BOOK_SCF_LAV >= (INT)(sizeof(FDKaacEnc_huff_ltabscf)/sizeof(FDKaacEnc_huff_ltabscf[0]))) {
-          isScale[sfb+sfboffs] = lastValIs + sizeof(FDKaacEnc_huff_ltabscf)/sizeof(FDKaacEnc_huff_ltabscf[0]) - CODE_BOOK_SCF_LAV - 1;
-      }
-      lastValIs = isScale[sfb+sfboffs];
-
       sfbEnergyRight[sfb+sfboffs] = FL2FXCONST_DBL(0.0f);
       sfbEnergyLdDataRight[sfb+sfboffs] = FL2FXCONST_DBL(-1.0f);
       sfbThresholdRight[sfb+sfboffs] = FL2FXCONST_DBL(0.0f);
